# -*- coding: utf-8 -*-

import math
import requests
import numpy as np

import uno
from com.sun.star.awt import MessageBoxButtons as MSG_BUTTONS

# https://wiki.documentfoundation.org/Macros/Python_Guide/Useful_functions
CTX = uno.getComponentContext()
SM = CTX.getServiceManager()
def create_instance(name, with_context=False):
    if with_context:
        instance = SM.createInstanceWithContext(name, CTX)
    else:
        instance = SM.createInstance(name)
    return instance


# https://wiki.documentfoundation.org/Macros/Python_Guide/Useful_functions
def msgbox(message, title='LibreOffice', buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox'):
    """ Create message box
        type_msg: infobox, warningbox, errorbox, querybox, messbox
        https://api.libreoffice.org/docs/idl/ref/interfacecom_1_1sun_1_1star_1_1awt_1_1XMessageBoxFactory.html
    """
    toolkit = create_instance('com.sun.star.awt.Toolkit')
    parent = toolkit.getDesktopWindow()
    mb = toolkit.createMessageBox(parent, type_msg, buttons, title, str(message))
    return mb.execute()


# https://answers.microsoft.com/en-us/msoffice/forum/all/is-there-a-message-box-that-doesnt-stop-a-macro/e47ea2ec-cbf0-48f4-96ab-0b1e578c1333
# https://forum.openoffice.org/en/forum/viewtopic.php?t=74660
# https://www.exceldemy.com/excel-vba-progress-bar-while-macro-running/

def call_service(service_name, data, api_key, server="127.0.0.1", port=5000, verify=True):
    url = server + ":" + str(port) + "/lactate/" + service_name
    response = requests.post(
        url, json = data,
        headers = {
            'x-api-key': api_key
            },
        verify = verify
        )
    return response


def call_param_service(workload, lactate, func, api_key, server="127.0.0.1", port=5000, verify=True):
    if isinstance(workload, np.ndarray):
        workload = workload.tolist()
    if isinstance(lactate, np.ndarray):
        lactate = lactate.tolist()
    
    data = {
        'workload': workload,
        'lactate': lactate,
        'func': func
        }        
    response = call_service("params", data, api_key, server, port, verify)
    
    #check response.status_code
    params = response.json()['params']
    err = response.json()['fit_error']

    return params, err


def call_eval_service(workload, func, params, api_key, server="127.0.0.1", port=5000, verify=True):
    if isinstance(workload, np.ndarray):
        workload = workload.tolist()
    
    data = {
        'workload': workload,
        'func': func,
        'params': params
        }        
    response = call_service("eval", data, api_key, server, port, verify)
    
    #check response.status_code
    lactate = response.json()['lactate']

    return lactate


def call_ltaer_service(workload, lactate, rest, method, func, api_key, server="127.0.0.1", port=5000, verify=True):
    if isinstance(workload, np.ndarray):
        workload = workload.tolist()
    if isinstance(lactate, np.ndarray):
        lactate = lactate.tolist()
    
    data = {
        'workload': workload,
        'lactate': lactate,
        'rest_lactate': rest,
        'method': method,
        'func': func
        }

    response = call_service("ltaer", data, api_key, server, port, verify)

    #check response.status_code
    aer = response.json()['aer']
    return aer
    

def call_ltan_service(workload, lactate, waer, method, func, api_key, server="127.0.0.1", port=5000, verify=True):
    if isinstance(workload, np.ndarray):
        workload = workload.tolist()
    if isinstance(lactate, np.ndarray):
        lactate = lactate.tolist()
    
    data = {
        'workload': workload,
        'lactate': lactate,
        'aer_workload': waer,
        'method': method,
        'func': func
        }

    response = call_service("ltan", data, api_key, server, port, verify)

    #check response.status_code
    an = response.json()['an']
    return an


def call_hr_param_service(workload, hr, func, api_key, server="127.0.0.1", port=5000, verify=True):
    if isinstance(workload, np.ndarray):
        workload = workload.tolist()
    if isinstance(hr, np.ndarray):
        hr = hr.tolist()
    
    data = {
        'workload': workload,
        'hr': hr,
        'func': func
        }        
    response = call_service("hr_params", data, api_key, server, port, verify)
    
    #check response.status_code
    params = response.json()['params']
    err = response.json()['fit_error']

    return params, err


def call_hr_eval_service(workload, func, params, api_key, server="127.0.0.1", port=5000, verify=True):
    if isinstance(workload, np.ndarray):
        workload = workload.tolist()
    
    data = {
        'workload': workload,
        'func': func,
        'params': params
        }        
    response = call_service("hr_eval", data, api_key, server, port, verify)
    
    #check response.status_code
    hr = response.json()['hr']

    return hr


def turn_everything_off(model=None):
    if model is None:
        model = XSCRIPTCONTEXT.getDocument()    
    model.lockControllers()
    model.addActionLock()    


def turn_everything_on(model=None):
    if model is None:
        model = XSCRIPTCONTEXT.getDocument()    
    model.removeActionLock()
    model.unlockControllers()


def ltweb_call(*args):
    # TODO: https://wiki.documentfoundation.org/Macros/Python_Guide/Calc/Calc_sheets
    # desktop = XSCRIPTCONTEXT.getDesktop()
    # model = desktop.getCurrentComponent()
    model = XSCRIPTCONTEXT.getDocument()
    verify = True

    data_sheet = model.Sheets.getByName("Data")
    settings_sheet = model.Sheets.getByName("settings")
    model_sheet = model.Sheets.getByName("model")
    chart_data_sheet = model.Sheets.getByName("chart_data")
    hr_model_sheet = model.Sheets.getByName("hr_model")
    hr_chart_data_sheet = model.Sheets.getByName("hr_chart_data")
    fixed_sheet = model.Sheets.getByName("fixed")

    server = settings_sheet.getCellRangeByName("B1").String
    port = int(settings_sheet.getCellRangeByName("B2").Value)
    api_key = settings_sheet.getCellRangeByName("B3").String

    last_data = int(model_sheet.getCellRangeByName("N2").Value)
    num_points = int(model_sheet.getCellRangeByName("N4").Value)
    wrange = data_sheet.getCellRangeByName("A2:A" + str(last_data)).getDataArray()
    lrange = data_sheet.getCellRangeByName("B2:B" + str(last_data)).getDataArray()    
    rest = float(data_sheet.getCellRangeByName("C2").Value)
    hrange = data_sheet.getCellRangeByName("D2:D" + str(last_data)).getDataArray()
    func = data_sheet.getCellRangeByName("I3").String
    hr_func = data_sheet.getCellRangeByName("J3").String
    aer_method = data_sheet.getCellRangeByName("G3").String
    an_method = data_sheet.getCellRangeByName("H3").String
    eval_wrange = chart_data_sheet.getCellRangeByName("A2:A" + str(2 + num_points)).getDataArray()
    fixed_last_data = int(fixed_sheet.getCellRangeByName("N1").Value)
    fixed_lrange = fixed_sheet.getCellRangeByName("A2:A" + str(fixed_last_data)).getDataArray()
            
    model_sheet.getCellRangeByName("Q1").String = func
    hr_model_sheet.getCellRangeByName("Q1").String = func

    workload = [x[0] for x in wrange]
    lactate = [x[0] for x in lrange]
    hr = [x[0] for x in hrange]
    eval_workload = [x[0] for x in eval_wrange]
    fixed_lactate = [x[0] for x in fixed_lrange]
    
    turn_everything_off(model)
    
    try:
        params, err = call_param_service(workload, lactate, func, api_key, server, port, verify=verify)
        hr_params, hr_err = call_hr_param_service(workload, hr, hr_func, api_key, server, port, verify=verify)
    except requests.exceptions.RequestException as e:
        turn_everything_on(model)
        message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
        msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
        return None
        
    model_sheet.getCellRangeByName("B3").Value = err
    hr_model_sheet.getCellRangeByName("B3").Value = hr_err

    try:
        eval_lactate = call_eval_service(eval_workload, func, params, api_key, server, port, verify=verify)
        eval_hr = call_hr_eval_service(eval_workload, hr_func, hr_params, api_key, server, port, verify=verify)
    except requests.exceptions.RequestException as e:
        turn_everything_on(model)    
        message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
        msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
        return None
    
    for i, (_lac, _hr) in enumerate(zip(eval_lactate, eval_hr)):
        chart_data_sheet.getCellRangeByName("B" + str(2+i)).Value = _lac
        hr_chart_data_sheet.getCellRangeByName("B" + str(2+i)).Value = _hr

    try:
        aer_wl = call_ltaer_service(workload, lactate, rest, aer_method, func, api_key, server, port, verify=verify)
    except requests.exceptions.RequestException as e:
        turn_everything_on(model)    
        message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
        msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
        return None
    
    if (aer_wl is None) or (math.isnan(aer_wl)):
        aer_wl = 0
        aer_lac = 0
        aer_hr = 0
        an_wl = 0
        an_lac = 0
        an_hr = 0
    else:
        try:
            aer_lac = call_eval_service([aer_wl], func, params, api_key, server, port, verify=verify)
            aer_hr = call_hr_eval_service([aer_wl], hr_func, hr_params, api_key, server, port, verify=verify)
        except requests.exceptions.RequestException as e:
            turn_everything_on(model)    
            message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
            msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
            return None

        aer_lac = aer_lac[0]
        aer_hr = aer_hr[0]
        
        try:
            an_wl = call_ltan_service(workload, lactate, aer_wl, an_method, func, api_key, server, port, verify=verify)        
        except requests.exceptions.RequestException as e:
            turn_everything_on(model)    
            message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
            msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
            return None

        if (an_wl is None) or (math.isnan(an_wl)):
            an_wl = 0
            an_lac = 0
            an_hr = 0
        else:
            try:
                an_lac = call_eval_service([an_wl], func, params, api_key, server, port, verify=verify)
                an_hr = call_hr_eval_service([an_wl], hr_func, hr_params, api_key, server, port, verify=verify)
            except requests.exceptions.RequestException as e:
                turn_everything_on(model)    
                message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
                msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
                return None
            an_lac = an_lac[0]
            an_hr = an_hr[0]
    model_sheet.getCellRangeByName("H2").Value = aer_wl
    model_sheet.getCellRangeByName("H3").Value = aer_lac
    hr_model_sheet.getCellRangeByName("H3").Value = aer_hr
    model_sheet.getCellRangeByName("H7").Value = an_wl
    model_sheet.getCellRangeByName("H8").Value = an_lac
    hr_model_sheet.getCellRangeByName("H8").Value = an_hr
    
    fixed_workload = []
    for _lac in fixed_lactate:
        # rest method finds the workload where lactate is 0.5 above rest lactate
        _lac05 = _lac - 0.5
        try:
            _wl05 = call_ltaer_service(workload, lactate, _lac05, "rest", func, api_key, server, port, verify=verify)
        except requests.exceptions.RequestException as e:
            turn_everything_on(model)    
            message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
            msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
            return None
        fixed_workload.append(_wl05)
    
    try:
        fixed_hr = call_hr_eval_service(fixed_workload, hr_func, hr_params, api_key, server, port, verify=verify)
    except requests.exceptions.RequestException as e:
        turn_everything_on(model)    
        message = "Error while accessing LT-web services. Please check your internet connection. Error details: " + str(e)
        msgbox(message, buttons=MSG_BUTTONS.BUTTONS_OK, type_msg='errorbox')
        return None

    for i, (_wl, _hr) in enumerate(zip(fixed_workload, fixed_hr)):
        fixed_sheet.getCellRangeByName("B" + str(2+i)).Value = _wl
        fixed_sheet.getCellRangeByName("C" + str(2+i)).Value = _hr

    
    turn_everything_on(model)    

    return None

    
g_exportedScripts = ltweb_call,

